import pandas as pd
import yaml

import sys 
from pathlib import Path
import ipynbname

# SQLAlchemy
from sqlalchemy import (
    create_engine, MetaData, Table, Column,
    String, DateTime, Integer, Boolean, Time, Date, Numeric, inspect, insert, text
)



import sys
from pathlib import Path
import ipynbname
import yaml
import shutil
import os

try: paths = Path(ipynbname.path()).parent
except: paths = Path(sys.argv[0]).parent

ENABLE_CONTEXT = False
try: # Context Locator
    for c in range(10):
        root = paths / "params.yaml" if c == 0 else paths.parents[c-1] / "params.yaml"
        if root.is_file():
            sys.path.append(str(root.parent))
            with open(root, "r", encoding="utf-8") as f: 
                parameters = yaml.safe_load(f); break;
    PARAMS = parameters['Settings']
    ENABLE_CONTEXT = True
except: PARAMS = {}

if ENABLE_CONTEXT:

    BASE_PATH = Path(PARAMS.get('DrdRoot'))
    SOURCE_PATH = BASE_PATH / 'src'
    SOURCE_PATH.mkdir(exist_ok=True)
    DATA_PATH = BASE_PATH / 'production' 
    DATA_PATH.mkdir(exist_ok=True)






try: from drilling.context import *
except: from context import *
import yaml

def ConPrm(name, context=PARAMS['ClientContext']):
    try:
        with open(BASE_PATH / 'src' / f"{context}.yaml", "r") as file:
            data = yaml.safe_load(file)
            data = data["DataSources"][name]
        return data
    except: ValueError(f"Context Source {name} not found in {context}.yaml")


from sqlalchemy import create_engine, text
from sshtunnel import SSHTunnelForwarder
try: from drilling.context_sources import *
except: from context_sources import *

class Connect():

    def __init__(self, parameters: dict):
        self.params = parameters

        # Banco
        self.db_dialect = self.params["Dialect"]
        self.db_protocol = self.params["Protocol"] + "://"
        self.db_host = self.params["Server"]
        self.db_port = self.params["Port"]
        self.db_user = self.params["User"]
        self.db_password = self.params["Pass"]
        self.db_name = self.params["Database"]
        self.db_driver = self.params["Driver"] or ""

        # SSH
        self.shh = self.params["SSH"]["Habilited"]
        self.ssh_host = self.params["SSH"].get("Host", "")
        self.ssh_port = self.params["SSH"].get("Port", 22)
        self.ssh_user = self.params["SSH"].get("User", "")
        self.ssh_password = str(self.params["SSH"].get("Pass", ""))

        # Objetos internos
        self.engine = None
        self.conn = None
        self.ssh_tunnel = None

    def __enter__(self):
        if self.shh:
            tunnel = SSHTunnelForwarder(
                (self.ssh_host, 22),
                ssh_username=self.ssh_user,
                ssh_password=self.ssh_password,
                remote_bind_address=(self.db_host, self.db_port),  # ajuste se necessário
                local_bind_address=('127.0.0.1', 0)
            )
            tunnel.start()
            self.ssh_tunnel = tunnel
            self.db_host = "127.0.0.1"
            self.db_port = self.ssh_tunnel.local_bind_port

        if self.db_dialect in ['PostgreSQL', 'SQLServer']:
            db_host = self.db_host
            db_port = self.db_port
            db_url = f"@{db_host}:{db_port}/"
            db_credentials = f"{self.db_user}:{self.db_password}"
            conn_str = f"{self.db_protocol}{db_credentials}{db_url}{self.db_name}{self.db_driver}"
            self.engine = create_engine(conn_str)
            self.conn = self.engine.connect()
            return self.conn
        

        else: raise ValueError(f"Dialect {self.db_dialect} not supported.")

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.conn: 
            self.conn.close()
        if self.engine: 
            self.engine.dispose()
        if self.ssh_tunnel:
            self.ssh_tunnel.stop()
        
if __name__ == "__main__":

    # Exemplo de uso para testes locais com sqlite em memória
    with Connect(ConPrm('Datalake_Devee')) as conn:
        result = conn.execute(text("SELECT 'success' as attempt"))
        for row in result:
            print(row)